import greenfoot.*;

import java.awt.Color;
import java.awt.Graphics;

/**
 * Counter that displays a number with a text prefix. Original author was Michael Kolling. Changes made by Nate Beasley
 * 
 * @author Nate Beasley
 * @version 1.5
 */
public class Counter extends Text
{
    private int value = 0;
    private int target = 0;

    /**
     * Makes a Counter with no text and a white font
     */
    public Counter()
    {
        this("", 0, Color.white);
    }
    
    /**
     * Makes a Counter with a custom text, with a white color.
     * @param prefix The text to be displayed
     */
    public Counter(String prefix)
    {
        this(prefix, 0, Color.white);
    }
    
    /**
     * Makes a Counter with custom text, and a custom value, with a white font.
     * @param prefix The text to be displayed
     * @param value The value of the counter
     */
    public Counter(String prefix, int value)
    {
        this(prefix, value, Color.white);
    }
    
    /**
     * Makes a Counter with custom text, value, and color.
     * @param prefix The text to be displayed
     * @param value The value of the counter
     * @param c The color of the text
     */
    public Counter(String prefix, int value, Color c)
    {
        super(prefix, c);
        target = value;
        updateImage();
    }
    
    /**
     * The Counter's act method, sees if the value of the counter needs to be added, or subtracted.
     */
    public void act() {
        if(value < target) {
            value++;
            updateImage();
        }
        else if(value > target) {
            value--;
            updateImage();
        }
    }

    /**
     * Adds an amount to the target value
     * @param score The amount to be added to the target value.
     */
    public void add(int score)
    {
        target += score;
    }

    /**
     * Subtracts an amount from the target value
     * @param score The amount to be subtracted from the target value.
     */
    public void subtract(int score)
    {
        target -= score;
    }
    
    /**
     * Sets the target value to a specific value
     * @param target The new target value
     */
    public void setTarget(int target)
    {
        this.target = target;
    }

    /**
     * @return The value of the Counter
     */
    public int getValue()
    {
        return value;
    }

    /**
     * Make the image
     */
    private void updateImage()
    {
        GreenfootImage image = getImage();
        image.clear();
        image.drawString(getText() + " " + value, 1, 12);
    }
}