import greenfoot.*;
import java.util.List;
import java.awt.Color;

/**
 * This world keeps track of all the balls, player, and score
 * 
 * @author Nate Beasley
 * @version 1.0.0
 */
public class DodgeWorld extends World
{
    private int timer;
    private int maxTime;
    private int speed;
    private Player player = new Player();
    private Counter count = new Counter("Score:", 0, java.awt.Color.black);

    /**
     * Constructor for objects of class DodgeWorld.
     */
    public DodgeWorld()
    {
        super(200, 200, 1, false);
        maxTime = 100;
        speed = 2;
        timer = Greenfoot.getRandomNumber(maxTime);
        addObject(player, getWidth()/2, getHeight()/2);
        addObject(count, 40, 10);
    }

    public void act()
    {
        updateVars();
        if (timer <= 0)
            releaseASquare();
        else
            timer--;
    }

    private void updateVars()
    {
        switch (count.getValue() / 10)
        {
            case 1: maxTime = 90;
            break;
            case 2: speed = 3;
            break;
            case 3: maxTime = 80;
            break;
            case 4: maxTime = 70;
            speed = 4;
            break;
            case 5: maxTime = 60;
            speed = 5;
            break;
            case 6: maxTime = 50;
            break;
            case 7: break;
            case 8: maxTime = 40;
            speed = 6;
            case 9: speed = 8;
            default: break;
        }
    }

    public void releaseASquare()
    {
        int x = Greenfoot.getRandomNumber(50)+200;
        if (Greenfoot.getRandomNumber(2)+1 == 1)
            x = -x;
        int y = Greenfoot.getRandomNumber(50)+200;
        if (Greenfoot.getRandomNumber(2)+1 == 1)
            y = -y;
        addObject(new Ball(speed, player), x, y);
        timer = Greenfoot.getRandomNumber(maxTime);
    }

    public void endEverything()
    {
        addHighScore();
        Greenfoot.stop();
    }

    private void addHighScore()
    {
        UserInfo ui_=UserInfo.getMyInfo();
        if (count.getValue() > ui_.getScore())
        {
            ui_.setScore(count.getValue());
            ui_.store();
        }
        if(ui_==null)
        {
            Actor a = new Actor(){};
            a.setImage(new GreenfootImage("Sorry,\nbut you are not logged in!",68,Color.white,new Color(0,0,0,0)));
            addObject(a,getWidth()/2,getHeight()/2);
            return;
        }

        List<UserInfo>
        list_ui_Top=UserInfo.getTop(7),
        list_ui_Nearby=UserInfo.getNearby(14);

        addObject(
            new Highscore(
                list_ui_Nearby, // All UserInfos that will be shown in the list
                200, //width of the highscore-field
                200, //height of the highscore-field
                5,  // maximal number of elements in the highscore
                ui_, // UserInfo of the current user
                new Color(100,100,255,200), // Highscore-background
                Color.black, // Highscore-frame
                new Color(100,0,0), // Highscore-text
                new Color(0,0,0,100), // Frame of the personal highscore-field
                new Color(200,200,200,150), // Field of the personal highscore
                new Color(0,0,0,100), // Frame of the general highscore-field
                new Color(100,100,100,150)), // Field of the general highscore
            getWidth()/2, getHeight()/2);
    }

    public Counter counter()
    {
        return count;
    }
}
