import java.util.List;
import java.util.ArrayList;
import greenfoot.*;
import java.awt.Color;
/**
 * This class easily creates a new highscore.
 * 
 * @author (Busch2207) 
 * @version (17.05.2013)
 */
public class Highscore extends Actor
{
    private int height,width,FieldNumber;
    private List<HighScoreField> L_HSF = new ArrayList();
    private Color
    col_Background,
    col_Frame;

    /** creates a new Top-highscore with the given listElements, width and height.*/
    public Highscore(int width,int height,int listFieldNumber)
    {
        this(width,height,listFieldNumber,true);
    }

    /** creates a new highscore with the given listElements, width and height. if the 'top' boolean ist setted true, it will take the top-scores, else a score of the persons, that are near the current User*/
    public Highscore(int width,int height,int listFieldNumber,boolean top)
    {
        this(top?UserInfo.getTop(listFieldNumber):UserInfo.getNearby(listFieldNumber),width,height,listFieldNumber,UserInfo.getMyInfo());
    }

    /** creates a new highscore with the given UserInfoList, width and height. In the last field ('listFieldNumber') is expected the maximal listSize. */
    public Highscore(List<UserInfo> LPD,int width,int height,int listFieldNumber)
    {
        this(LPD,width,height,listFieldNumber,UserInfo.getMyInfo());
    }

    /** creates a new highscore with the given UserInfoList, width and height. In the last field is expected the UserInfo of the User, which should be highlighted.*/
    public Highscore(List<UserInfo> LPD,int width,int height,int listFieldNumber,UserInfo ui_MyData)
    {
        this(LPD,width,height,listFieldNumber,ui_MyData,
            new Color(100,100,100,200), // Highscore-background
            new Color(0,0,0,150), // Highscore-frame
            Color.white, // Highscore-text
            new Color(0,0,0,100), // Frame of the personal highscore-field
            new Color(200,200,200,150), // Field of the personal highscore
            new Color(0,0,0,100), // Frame of the general highscore-field
            new Color(100,100,100,150)); // Field of the general highscore
    }

    /** Creates a new Highscore with the given colors.
     * The order of the Colors is: Highscore-Background / Highscore-Frame / String-Color / Color of the highlighted Score-Field-Frame / Color of the highlighted Score-Field / Color of the general Score-Field-Frame / Color of the general Score-Field */
    public Highscore(List<UserInfo> LPD,int width,int height,int listFieldNumber,UserInfo ui_MyData,Color col_Background,Color col_Frame,Color col_String,Color col_MyScoreFrame,Color col_MyScoreField,Color col_GeneralScoreFrame,Color col_GeneralScoreField)
    {
        this.FieldNumber=listFieldNumber;
        this.col_Background=col_Background;
        this.col_Frame=col_Frame;
        this.height=height;
        this.width=width;
        if(LPD!=null)
        {
            int i_Count=0;
            for(UserInfo PD : LPD)
            {
                L_HSF.add(new HighScoreField(PD,ui_MyData,width-2,(height-2)/listFieldNumber,col_String,col_MyScoreFrame,col_MyScoreField,col_GeneralScoreFrame,col_GeneralScoreField));
                if(++i_Count>=FieldNumber)
                    break;
            }
        }
        updateImage();
    }

    private void updateImage()
    {
        GreenfootImage i = new GreenfootImage(width,height);
        i.setColor(col_Background);
        i.fillRect(1,1,width-2,((height-2)/FieldNumber)*FieldNumber);
        i.setColor(col_Frame);
        i.drawRect(0,0,width-1,height-1);
        i.fillRect(1,((height-2)/FieldNumber)*FieldNumber+1,width-2,height-((height-2)/FieldNumber)*FieldNumber-2);

        int y = 1;
        for(HighScoreField HSF : L_HSF)
        {
            GreenfootImage i2 = HSF.getImage();
            i.drawImage(i2,1,y);
            y+=i2.getHeight();
        }
        setImage(i);
    }

    private static Color col_Invisible=new Color(0,0,0,0);

    private static Color getOppositeColorNoteContrast(Color C)
    {
        int
        r=C.getRed(),
        g=C.getGreen(),
        b=C.getBlue(),
        r2=255-r,
        g2=255-g,
        b2=255-b;
        return Math.abs(r-r2)<50&&Math.abs(b-b2)<50&&Math.abs(r-r2)<50?new Color(r-100,g-100,b-100):new Color(r2,g2,b2);
    }

    private static GreenfootImage scaleToMax(GreenfootImage gi_Image, int width)
    {
        return scaleToMax(gi_Image, width, width);
    }

    private static GreenfootImage scaleToMax(GreenfootImage gi_Image, int width,int height)
    {
        gi_Image=new GreenfootImage(gi_Image);
        int w = gi_Image.getWidth(),h = gi_Image.getHeight();
        if(w<width && h<height)
        {
            return gi_Image;
        }
        double precent;
        if(w>h)
            precent = (double)width/(double)w;
        else
            precent = (double)height/(double)h;
        gi_Image.scale((int)(precent*w),(int)(precent*h));
        return gi_Image;
    }

    private static GreenfootImage getTextMaxWidth(int width, String S,int startsize,Color col_String)
    {
        return getTextMaxWidth(width,S,startsize,col_String,col_Invisible);
    }

    private static GreenfootImage getTextMaxWidth(int width, String S,int startsize,Color col_String,Color col_Background)
    {
        for(;startsize>0; startsize--)
        {
            GreenfootImage i = new GreenfootImage(S,startsize,col_String,col_Background);
            if(i.getWidth()<=width)
                return i;
        }
        return null;
    }

    private static GreenfootImage
    GI_MyParts,
    GI_Field;
    private static int i_hsf_height=3, i_hsf_width=3;
    private static boolean LastEqual=false;

    private class HighScoreField
    {
        private Color
        col_String,
        col_MyScoreFrame,
        col_MyScore,
        col_ScoreFrame,
        col_Score;
        private UserInfo
        PD_Data,
        PD_MyData;
        private GreenfootImage GI_Image;

        public HighScoreField(UserInfo PD_Data,UserInfo PD_MyData,int i_hsf_width,int i_hsf_height,Color col_String,Color col_MyScoreFrame,Color col_MyScore,Color col_ScoreFrame,Color col_Score)
        {
            this.col_String=col_String;
            this.col_MyScoreFrame=col_MyScoreFrame;
            this.col_MyScore=col_MyScore;
            this.col_ScoreFrame=col_ScoreFrame;
            this.col_Score=col_Score;
            this.PD_Data=PD_Data;
            this.PD_MyData=PD_MyData;
            scale(i_hsf_height,i_hsf_width);
            UpdateImage();
        }

        private void scale(int i_i_hsf_height, int i_i_hsf_width)
        {
            if(i_hsf_height==i_i_hsf_height && i_hsf_width == i_i_hsf_width && LastEqual == (PD_MyData != null && PD_Data.getUserName().equals(PD_MyData.getUserName())))
                return;
            LastEqual = (PD_MyData != null && PD_Data.getUserName().equals(PD_MyData.getUserName()));
            i_hsf_height=i_i_hsf_height;
            i_hsf_width=i_i_hsf_width;
            GI_Field = new GreenfootImage(i_hsf_width,i_hsf_height);
            GI_Field.setColor((PD_MyData != null && PD_Data.getUserName().equals(PD_MyData.getUserName()) ? col_MyScoreFrame : col_ScoreFrame));
            GI_Field.drawRect(0,0,i_hsf_width-1,i_hsf_height-1);
            GI_Field.setColor((PD_MyData != null && PD_Data.getUserName().equals(PD_MyData.getUserName()) ? col_MyScore : col_Score));
            GI_Field.fillRect(1,1,i_hsf_width-2,i_hsf_height-2);
            GI_Field.setColor(getOppositeColorNoteContrast(col_String));
            GI_Field.fillRect(i_hsf_width*4/5,1,i_hsf_width*1/5,i_hsf_height-2);
        }

        public void UpdateImage()
        {
            GreenfootImage i = new GreenfootImage(GI_Field),
            gi_Rank      = getTextMaxWidth(i_hsf_width*1/10-2,""+PD_Data.getRank(),(int)(i_hsf_height*0.75),col_String),
            gi_UserImage = scaleToMax(PD_Data.getUserImage(),i_hsf_height-2),
            gi_UserName  = getTextMaxWidth((int)(i_hsf_width*0.45),PD_Data.getUserName(),i_hsf_height*2/3,col_String),
            gi_UserScore = getTextMaxWidth(i_hsf_width*2/10-2,PD_Data.getScore()+"",i_hsf_height*2/3,col_String);
            i.drawImage(gi_Rank     ,i_hsf_width*1/20-gi_Rank.getWidth()/2       ,i_hsf_height/2-gi_Rank     .getHeight()/2);
            i.drawImage(gi_UserImage,i_hsf_width*1/10                            ,i_hsf_height/2-gi_UserImage.getHeight()/2);
            i.drawImage(gi_UserName ,(i_hsf_width*7/10-gi_UserImage.getWidth()+2)/2+i_hsf_width*1/10+gi_UserImage.getWidth()-gi_UserName.getWidth()/2,i_hsf_height/2-gi_UserName .getHeight()/2);
            i.drawImage(gi_UserScore,i_hsf_width*9/10-gi_UserScore.getWidth()/2+1,i_hsf_height/2-gi_UserScore.getHeight()/2);
            this.GI_Image=i;
        }

        public GreenfootImage getImage()
        {
            return new GreenfootImage(GI_Image);
        }
    }
}
