import greenfoot.*;

import java.awt.Color;
import java.awt.Graphics;

/**
 * Text is an actor that displays text of a specific color on the screen. Based off the original Text class by Michael Kolling
 * 
 * @author Nate Beasley
 * @version 0.1
 */
public class Text extends Actor
{
    private Color textColor;
    
    private String text;
    private int stringLength;
    private int size;

    /**
     * Makes a Text with no text and a white font
     */
    public Text()
    {
        this("", Color.white);
    }
    
    /**
     * Makes a Text with a specific font size.
     * @param prefix The text to be displayed
     * @param c The Color
     * @param size Font size
     */
    public Text(String prefix, Color c, int size)
    {
        textColor = c;
        this.size = size;
        
        text = prefix;
        
        setImage(new GreenfootImage(text, size * 10, Color.WHITE, Color.WHITE));
        GreenfootImage image = getImage();
        image.setColor(textColor);
        updateImage();
    }

    /**
     * Makes a Text with a custom text, and a custom color
     * @param prefix The text that will be displayed
     * @param c The color of the text
     */
    public Text(String prefix, Color c)
    {
        textColor = c;
        
        text = prefix;
        stringLength = (text.length() + 2) * 10;

        setImage(new GreenfootImage(stringLength, 16));
        GreenfootImage image = getImage();
        image.setColor(textColor);
        updateImage();
    }
    
    /**
     * Makes a Text with a custom text, with a white color.
     * @param prefix The text to be displayed
     */
    public Text(String prefix)
    {
        this(prefix, Color.white);
    }
    
    /**
     * @return The text of the Text
     */
    public String getText()
    {
        return text;
    }
    
    /**
     * Sets the prefix to a new String
     * @param text The new prefix
     */
    public void setText(String text)
    {
        this.text = text;
        updateImage();
    }

    /**
     * Make the image
     */
    private void updateImage()
    {
        GreenfootImage image = getImage();
        image.clear();
        image.drawString(text, 1, 12);
    }   
}
